#
#   dupreveal.py
#
#   Author  : Ahmed Obied (ahmed.obied@gmail.com)
#   Date    : 10/2009
#   Version : 0.1
#
#   DupReveal is a tool to identify duplicate files in a given folder. The tool
#   groups the paths of the duplicate files in the output file.
#
#   Usage: ./dupreveal.py [path to folder] [output file name]
#   

import sys
import os
import md5
import threading
import Queue
 
TOTAL_THREAD_NUM = 30

file_queue = Queue.Queue()
lock = threading.Lock()
hash_db = {}
ignored = 0
total_size = 0.0
sep = '/'

class WorkerThread(threading.Thread):

    def __init__(self):
        threading.Thread.__init__(self)

    def run(self):
        global ignored, total_size
        while True:
            cfile = file_queue.get()
            data = None
            try:
                fd = open(cfile, 'rb')
                st = os.stat(cfile)
                total_size += st.st_size
                data = fd.read()
                fd.close()
            except:
                lock.acquire()
                ignored += 1
                lock.release()
                file_queue.task_done()
                continue
            hash_value = md5.new()
            hash_value.update(data)
            digest = hash_value.hexdigest()
            lock.acquire()
            if hash_db.has_key(digest):
                hash_db[digest] += [cfile]
            else:
                hash_db[digest] = [cfile]
            lock.release()
            file_queue.task_done()
            
def start_threads():
    for i in range(TOTAL_THREAD_NUM):
        worker_thread = WorkerThread()
        worker_thread.setDaemon(True)
        worker_thread.start()

def get_files(folder):
    print '[-] Scanning "%s" ...' % folder
    for path, dirs, files in os.walk(folder):
        for cfile in files:
            file_queue.put(path + sep + cfile)

def dump_data(output_file):
    fd = open(output_file, 'w')
    for key in hash_db.keys():
        if len(hash_db[key]) > 1:
            fd.write('\r\n')
            for cfile in hash_db[key]:
                fd.write('%s\r\n' % cfile)
    fd.close()

def convert_size():
    total_bsize = total_size
    c_table = ['B', 'KB', 'MB', 'GB', 'TB']
    i = 0
    while True:
        tmp = total_bsize / 1024.0
        if tmp >= 1: 
            i += 1
            total_bsize = tmp
        else:
            break
    return total_bsize, c_table[i]
       
def main():
    if len(sys.argv[1:]) != 2:
        print 'Usage: %s [path to folder] [output file name]' % sys.argv[0]  
        sys.exit(0)
    print
    print '[**********************************]'
    print '[   DupReveal 0.1 by Ahmed Obied   ]'
    print '[**********************************]'
    print 
    if sys.platform == 'win32':
        global sep
        sep = '\\'
    folder = sys.argv[1]
    output_file = sys.argv[2] + '.txt'
    get_files(folder)
    total_files = file_queue.qsize()
    start_threads()
    file_queue.join()
    diff = (total_files - ignored) - len(hash_db) 
    total_bsize, bytes = convert_size()
    print
    print '[-] Total size            : %.2f %s' % (total_bsize, bytes)
    print '[-] Total number of files : %d' % total_files  
    print '[-] Files ignored         : %d' % ignored
    print '[-] Files analyzed        : %d' % (total_files - ignored)
    print '[-] Unique files          : %d' % len(hash_db)
    print '[-] Duplicate files       : %d' % diff   
    if diff:
        dump_data(output_file)   
        print
        print '[-] Results are in "%s"' % (os.getcwd() + sep + output_file)
    print
    
if __name__ == '__main__':
    main()
    
